@@ -2,6 +2,7 @@
 #include "gmock/gmock.h"
 #include <canopen_master/StateMachine.hpp>
 #include <canopen_master/SDO.hpp>
+#include <canopen_master/Objects.hpp>
 
 using namespace std;
 using namespace canopen_master;
@@ -73,7 +74,9 @@ TEST(StateMachine, processThrowsOnAnEmergencyMessage) {
     msg.can_id = 0x082;
     msg.data[0] = 0x10;
     msg.data[1] = 0x10;
+    msg.data[2] = 0xFA;
     ASSERT_THROW(machine.process(msg), EmergencyMessageReceived);
+    ASSERT_EQ(0xFA, machine.get<uint8_t>(ErrorRegister::OBJECT_ID, ErrorRegister::OBJECT_SUB_ID));
 }
 
 TEST(StateMachine, processDoesNotThrowOnAnEmergencyWithZeroCode) {
@@ -156,6 +159,19 @@ TEST(StateMachine, download_32) {
     );
 }
 
+
+TEST(StateMachine, download_unknown) {
+    StateMachine machine(2, true);
+    std::vector<uint8_t> raw { 0xFE, 0x00, 0x00, 0x00 };
+    canbus::Message msg = machine.download(0x1801, 3, raw.data(), raw.size());
+    ASSERT_EQ(msg.can_id, 0x602);
+    ASSERT_EQ(msg.size, 8);
+    EXPECT_THAT(
+        std::vector<uint8_t>(msg.data, msg.data + 8),
+        ElementsAre(0x22, 0x01, 0x18, 0x03, 0xFE, 0x00, 0x00, 0x00)
+    );
+}
+
 TEST(StateMachine, set_and_get) {
     StateMachine machine(2);
     uint16_t value = 0x1234;
@@ -165,6 +181,32 @@ TEST(StateMachine, set_and_get) {
     ASSERT_EQ(time, machine.timestamp(0x12, 0x1));
 }
 
+TEST(StateMachine, getDefinesSizeOfObject) {
+    StateMachine machine(2, true);
+    canbus::Message msg;
+    msg.time = base::Time::now();
+    msg.can_id = 0x582;
+    msg.data[0] = 0x42;
+    msg.data[1] = 0x01;
+    msg.data[2] = 0x18;
+    msg.data[3] = 0x03;
+    msg.data[4] = 0xEF;
+    msg.data[5] = 0xBE;
+    msg.data[6] = 0xFF;
+    msg.data[7] = 0xFF;
+    machine.process(msg);
+    ASSERT_EQ(0xBEEF, machine.get<uint16_t>(0x1801, 3));
+    ASSERT_EQ(0xBEEF, machine.get<uint16_t>(0x1801, 3));
+    ASSERT_THROW(machine.get<uint32_t>(0x1801, 3), InvalidObjectType);
+    ASSERT_THROW(machine.get<uint8_t>(0x1801, 3), InvalidObjectType);
+}
+
+TEST(StateMachine, setRejectsZeroUpdateTime) {
+    StateMachine machine(2);
+    uint16_t value = 0x1234;
+    ASSERT_THROW(machine.set(0x12, 0x1, value, base::Time()), std::invalid_argument);
+}
+
 TEST(StateMachine, getFailsOnADeclaredButUnreadObject) {
     StateMachine machine(2);
     machine.declare(0x1801, 3, 2);
@@ -187,7 +229,9 @@ TEST(StateMachine, downloadOfADeclaredObject) {
 TEST(StateMachine, downloadFailsIfDeclaredSizeMismatches) {
     StateMachine machine(2);
     machine.declare(0x1801, 3, 4);
-    ASSERT_THROW(machine.download(0x1801, 3, static_cast<uint16_t>(0)), ObjectSizeMismatch);
+    ASSERT_THROW(
+        machine.download(0x1801, 3, static_cast<uint16_t>(0)),
+        ObjectSizeMismatch);
 }
 
 TEST(StateMachine, processUploadReply) {
@@ -207,6 +251,63 @@ TEST(StateMachine, processUploadReply) {
     ASSERT_EQ(0x3FE, machine.get<uint16_t>(0x1801, 3));
 }
 
+TEST(StateMachine, processUnknownSizeUploadReply) {
+    StateMachine machine(2, true);
+    canbus::Message msg;
+    msg.time = base::Time::now();
+    msg.can_id = 0x582;
+    msg.data[0] = 0x42;
+    msg.data[1] = 0x01;
+    msg.data[2] = 0x18;
+    msg.data[3] = 0x03;
+    msg.data[4] = 0xFE;
+    msg.data[5] = 0x03;
+    msg.data[6] = 0xFF;
+    msg.data[7] = 0xFF;
+    ASSERT_EQ(Update(StateMachine::PROCESSED_SDO, 0x1801, 3), machine.process(msg));
+    ASSERT_EQ(0x3FE, machine.get<uint16_t>(0x1801, 3));
+
+    // It should allow receiving with size bits unset even if we already know the size
+    ASSERT_EQ(Update(StateMachine::PROCESSED_SDO, 0x1801, 3), machine.process(msg));
+}
+
+TEST(StateMachine, processUnknownSizeUploadReplyOfAnAlreadyDefinedObject) {
+    StateMachine machine(2, true);
+    canbus::Message msg;
+    msg.time = base::Time::now();
+    msg.can_id = 0x582;
+    msg.data[0] = 0x42;
+    msg.data[1] = 0x01;
+    msg.data[2] = 0x18;
+    msg.data[3] = 0x03;
+    msg.data[4] = 0xFE;
+    msg.data[5] = 0x03;
+    msg.data[6] = 0xFF;
+    msg.data[7] = 0xFF;
+    machine.process(msg);
+
+    msg.data[4] = 0xEF;
+    msg.data[5] = 0xBE;
+    machine.process(msg);
+    ASSERT_EQ(0xBEEF, machine.get<uint16_t>(0x1801, 3));
+}
+
+TEST(StateMachine, processUploadReplyRejectsZeroUpdateTime) {
+    StateMachine machine(2);
+    canbus::Message msg;
+    msg.time = base::Time();
+    msg.can_id = 0x582;
+    msg.data[0] = 0x4B;
+    msg.data[1] = 0x01;
+    msg.data[2] = 0x18;
+    msg.data[3] = 0x03;
+    msg.data[4] = 0xFE;
+    msg.data[5] = 0x03;
+    msg.data[6] = 0x00;
+    msg.data[7] = 0x00;
+    ASSERT_THROW(machine.process(msg), ProtocolError);
+}
+
 TEST(StateMachine, processSDOAbort)
 {
     canbus::Message msg;
